unit ufUrlDecode;
{$INCLUDE 'Plugin.inc'}

interface

uses
  SysUtils, Classes,
  uXml,
  uFilter, uMessages;

type
  TFilterUrlDecode = class(TFilter)
  public
    class function ID: string; override;
    class function Name: string; override;
    class function Priority: TFilter.TPriority; override;
    class function AllowUnconfigured: boolean; override;
    procedure Clear; override;
    function IsSupported(const FileName, Content: string): boolean; override;
    function Execute(const FileName, Content: string): string; override;
  end;

implementation

uses
  SynaCode;

{ TFilterUrlDecode }

class function TFilterUrlDecode.AllowUnconfigured: boolean;
begin
  Result := True;
end;

procedure TFilterUrlDecode.Clear;
begin
  inherited;
  Description := GetMsg(MFilterUrlDecodeDescription);
  FileMasks.Add('*.url');
end;

function TFilterUrlDecode.Execute(const FileName, Content: string): string;
var
  Decoded: AnsiString;
  i: integer;
begin
  Decoded := {$IFDEF UNICODE} AnsiString {$ENDIF} (Content);
  for i := 1 to Length(Decoded) do
  begin
    case Decoded[i] of
      '+':
        Decoded[i] := ' ';
    end;
  end;
  Decoded := DecodeUrl(Decoded);
  Result := {$IFDEF UNICODE} string {$ENDIF} (Decoded);
end;

class function TFilterUrlDecode.ID: string;
begin
  Result := 'urldec';
end;

function TFilterUrlDecode.IsSupported(const FileName, Content: string): boolean;

  function IsHex(C: Char): boolean;
  begin
    Result := CharInSet(C, ['0'..'9', 'a'..'f', 'A'..'F']);
  end;

var
  i, n: integer;
begin
  Result := False;
  if inherited IsSupported(FileName, Content) then
  begin
    n := Length(Content);
    for i := 1 to n do
    begin
      if Content[i] = '%' then
      begin
        if (i + 2) < n then
          Exit;
        if not IsHex(Content[i+1]) then
          Exit;
        if not IsHex(Content[i+2]) then
          Exit;
      end;
    end;
    Result := True;
  end;
end;

class function TFilterUrlDecode.Name: string;
begin
  Result := GetMsg(MFilterUrlDecodeName);
end;

class function TFilterUrlDecode.Priority: TFilter.TPriority;
begin
  Result := prLow;
end;

initialization
  TRegisteredFilters.Add(TFilterUrlDecode);

end.
