unit uOptions;
{$INCLUDE 'Plugin.inc'}

interface

uses
  SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uMessages;

type
  TOptions = class(TFarConfig)
  private
    fFilterKeys: TStringList;
    function GetFilterKey(Index: integer): string;
    function GetFilterKeyCount: integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Reload;
    procedure DeleteFilter(const Key: string);
    procedure DeleteAllFilters;
    function GetFilterKeys: TStringList;
    property FilterKeyCount: integer read GetFilterKeyCount;
    property FilterKeys[Index: integer]: string read GetFilterKey;
  end;

  TFilterOptions = class(TFarConfig)
  private
    fKey: string;
    fMustExist: boolean;
    function GetDescription: string;
    function GetFileMaskStr: string;
    function GetID: string;
    procedure SetDescription(const Value: string);
    procedure SetFileMaskStr(const Value: string);
    procedure SetID(const Value: string);
    procedure SetKey(const Value: string);
    procedure SetMustExist(const Value: boolean);
    function GetEditBeforeUse: boolean;
    procedure SetEditBeforeUse(Value: boolean);
  protected
    procedure CheckActive;
  public
    constructor Create;
    property Key: string read fKey write SetKey;
    property MustExist: boolean read fMustExist write SetMustExist;
    property ID: string read GetID write SetID;
    property Description: string read GetDescription write SetDescription;
    property FileMaskStr: string read GetFileMaskStr write SetFileMaskStr;
    property EditBeforeUse: boolean read GetEditBeforeUse write SetEditBeforeUse;
  end;

implementation

const
  CONFIG_ROOT = 'Filters';
  FILTERS_ROOT = '\Filters';

{ TOptions }

constructor TOptions.Create;
begin
  inherited Create(CONFIG_ROOT, True);
  fFilterKeys := TStringList.Create;
  Reload;
end;

procedure TOptions.DeleteAllFilters;
begin
  Close;
  try
    if Open('') then
      DeleteKey(FILTERS_ROOT);
  finally
    Reload;
  end;
end;

procedure TOptions.DeleteFilter(const Key: string);
begin
  Close;
  try
    if Open(FILTERS_ROOT) then
      DeleteKey(Key);
  finally
    Reload;
  end;
end;

destructor TOptions.Destroy;
begin
  FreeAndNil(fFilterKeys);
  inherited;
end;

function TOptions.GetFilterKey(Index: integer): string;
begin
  Result := fFilterKeys[Index];
end;

function TOptions.GetFilterKeyCount: integer;
begin
  Result := fFilterKeys.Count;
end;

function TOptions.GetFilterKeys: TStringList;
begin
  Result := TStringList.Create;
  Result.Assign(fFilterKeys);
end;

procedure TOptions.Reload;
begin
  Close;
  fFilterKeys.Clear;
  try
    if Open(FILTERS_ROOT) then
    begin
      ReadKeyNames(fFilterKeys);
      fFilterKeys.Sort;
    end;
  finally
    Close;
  end;
end;

{ TFilterOptions }

procedure TFilterOptions.CheckActive;
begin
  if not IsOpen then
    raise EFarError.Create(GetMsg(MConfigNotActive));
end;

constructor TFilterOptions.Create;
begin
  inherited Create(CONFIG_ROOT, False);
  fKey := '';
  fMustExist := True;
end;

function TFilterOptions.GetDescription: string;
begin
  CheckActive;
  Result := ReadString('Description');
end;

function TFilterOptions.GetEditBeforeUse: boolean;
begin
  CheckActive;
  Result := ReadBoolean('EditBeforeUse');
end;

function TFilterOptions.GetFileMaskStr: string;
begin
  CheckActive;
  Result := ReadString('FileMasks');
end;

function TFilterOptions.GetID: string;
begin
  CheckActive;
  Result := ReadString('FilterID');
end;

procedure TFilterOptions.SetDescription(const Value: string);
begin
  CheckActive;
  WriteString('Description', Value);
end;

procedure TFilterOptions.SetEditBeforeUse(Value: boolean);
begin
  CheckActive;
  WriteBoolean('EditBeforeUse', Value);
end;

procedure TFilterOptions.SetFileMaskStr(const Value: string);
begin
  CheckActive;
  WriteString('FileMasks', Value);
end;

procedure TFilterOptions.SetID(const Value: string);
begin
  CheckActive;
  WriteString('FilterID', Value);
end;

procedure TFilterOptions.SetKey(const Value: string);
begin
  Close;
  if Value <> '' then
  begin
    if not Open(FILTERS_ROOT) then
      raise EFarError.CreateFmt(GetMsg(MConfigNotFound), [Value]);
    if MustExist and (not KeyExists(Value)) then
      raise EFarError.CreateFmt(GetMsg(MConfigNotFound), [Value]);
    if not Open(FILTERS_ROOT + '\' + Value) then
      raise EFarError.CreateFmt(GetMsg(MConfigNotFound), [Value]);
  end;
  fKey := Value;
end;

procedure TFilterOptions.SetMustExist(const Value: boolean);
begin
  fMustExist := Value;
  Key := Key;
end;

end.
